
#include "DirectSound.h"
#include "../Framework/Debug.h"
#include "../Framework/DirectX.h"

DirectSound::DirectSound()
{
	m_pWaveData = 0;
	m_waveSize	= 0;
	m_pDSBuffer = NULL;
	m_pDS8 = NULL;
}

DirectSound::~DirectSound()
{
	SAFE_RELEASE(m_pDSBuffer);
	SAFE_RELEASE(m_pDS8);
}

//-------------------------
//	  Wavet@CJ
//-------------------------
bool DirectSound::OpenWaveFile(LPWSTR FileName, WAVEFORMATEX &waveFormatEx, char** ppData, DWORD &dataSize )
{
	if ( FileName == 0 )
                return false;

        HMMIO hMmio = NULL;
        MMIOINFO mmioInfo;

        // Wavet@CI[v
        memset( &mmioInfo, 0, sizeof(MMIOINFO) );
        hMmio = mmioOpen( FileName, &mmioInfo, MMIO_READ );
        if( !hMmio )
                return false; // t@CI[vs

        // RIFF`N
        MMRESULT mmRes;
        MMCKINFO riffChunk;
        riffChunk.fccType = mmioFOURCC('W', 'A', 'V', 'E');
        mmRes = mmioDescend( hMmio, &riffChunk, NULL, MMIO_FINDRIFF );
        if( mmRes != MMSYSERR_NOERROR ) {
                mmioClose( hMmio, 0 );
                return false;
        }

        // tH[}bg`N
        MMCKINFO formatChunk;
        formatChunk.ckid = mmioFOURCC('f', 'm', 't', ' ');
        mmRes = mmioDescend( hMmio, &formatChunk, &riffChunk, MMIO_FINDCHUNK );
        if( mmRes != MMSYSERR_NOERROR ) {
                mmioClose( hMmio, 0 );
                return false;
        }
        DWORD fmsize = formatChunk.cksize;
        DWORD size = mmioRead( hMmio, (HPSTR)&waveFormatEx, fmsize );
        if( size != fmsize ) {
                mmioClose( hMmio, 0 );
                return false;
        }

        mmioAscend( hMmio, &formatChunk, 0 );

        // f[^`N
        MMCKINFO dataChunk;
        dataChunk.ckid = mmioFOURCC('d', 'a', 't', 'a');
        mmRes = mmioDescend( hMmio, &dataChunk, &riffChunk, MMIO_FINDCHUNK );
        if( mmRes != MMSYSERR_NOERROR ) {
                mmioClose( hMmio, 0 );
                return false;
        }
        *ppData = new char[ dataChunk.cksize ];
        size = mmioRead( hMmio, (HPSTR)*ppData, dataChunk.cksize );
        if(size != dataChunk.cksize ) {
                delete[] *ppData;
                return false;
        }
        dataSize = size;

        // nhN[Y
        mmioClose( hMmio, 0 );

        return true;
}

//-------------------------
//	 	  ǂݍ
//-------------------------
bool DirectSound::Load( LPWSTR FileName )
{
	// Wavet@C̓ǂݍ
	if( !OpenWaveFile( FileName, m_wFmt, &m_pWaveData, m_waveSize ) )
	{
		MessageBox(NULL,L"Waveǂݍݎs",L"Error",MB_OK);
		// sAfalseԂ
		return false;
	}

	DirectSoundCreate8(NULL, &m_pDS8, NULL);
#ifdef _DEBUG
	m_pDS8->SetCooperativeLevel(Debug::Instance()->hWnd, DSSCL_PRIORITY);
#else
	m_pDS8->SetCooperativeLevel(DirectX::Instance()->hWnd, DSSCL_PRIORITY);
#endif
	m_DSBufferDesc.dwSize = sizeof(DSBUFFERDESC);
	m_DSBufferDesc.dwFlags = DSBCAPS_CTRLVOLUME;
	m_DSBufferDesc.dwBufferBytes = m_waveSize;
	m_DSBufferDesc.dwReserved = 0;
	m_DSBufferDesc.lpwfxFormat = &m_wFmt;
	m_DSBufferDesc.guid3DAlgorithm = GUID_NULL;

	IDirectSoundBuffer *ptmpBuf = 0;
	m_pDS8->CreateSoundBuffer( &m_DSBufferDesc, &ptmpBuf, NULL );
	ptmpBuf->QueryInterface( IID_IDirectSoundBuffer8 ,(void**)&m_pDSBuffer );
	ptmpBuf->Release();
	if ( m_pDSBuffer == 0 ) {

		// ZJ_obt@̉
		SAFE_RELEASE(m_pDS8);

		// Wavef[^̉
		SAFE_DELETE_ARRAY(m_pWaveData);

		return false;
	}

	// ZJ_obt@Wavef[^
	LPVOID lpvWrite = 0;
	DWORD dwLength = 0;
	if ( DS_OK == m_pDSBuffer->Lock( 0, 0, &lpvWrite, &dwLength, NULL, NULL, DSBLOCK_ENTIREBUFFER ) ) {
		memcpy( lpvWrite, m_pWaveData, dwLength);
		m_pDSBuffer->Unlock( lpvWrite, dwLength, NULL, 0);
	}
	SAFE_DELETE_ARRAY(m_pWaveData);

	return true;
}

//-------------------------
//	 	    Đ
//-------------------------
void DirectSound::Play()
{
	m_pDSBuffer->SetVolume(-100);
	m_pDSBuffer->SetCurrentPosition(0);
	m_pDSBuffer->Play( 0, 0, 0 );
}

//-------------------------
//	 	    ~
//-------------------------
void DirectSound::Stop()
{
	m_pDSBuffer->Stop();
}